from typing import Any

COM_SLEEP = ...  # type: Any
COM_QUIT = ...  # type: Any
COM_INIT_DB = ...  # type: Any
COM_QUERY = ...  # type: Any
COM_FIELD_LIST = ...  # type: Any
COM_CREATE_DB = ...  # type: Any
COM_DROP_DB = ...  # type: Any
COM_REFRESH = ...  # type: Any
COM_SHUTDOWN = ...  # type: Any
COM_STATISTICS = ...  # type: Any
COM_PROCESS_INFO = ...  # type: Any
COM_CONNECT = ...  # type: Any
COM_PROCESS_KILL = ...  # type: Any
COM_DEBUG = ...  # type: Any
COM_PING = ...  # type: Any
COM_TIME = ...  # type: Any
COM_DELAYED_INSERT = ...  # type: Any
COM_CHANGE_USER = ...  # type: Any
COM_BINLOG_DUMP = ...  # type: Any
COM_TABLE_DUMP = ...  # type: Any
COM_CONNECT_OUT = ...  # type: Any
COM_REGISTER_SLAVE = ...  # type: Any
