from typing import Any, IO

version = ...  # type: int

def dump(value: Any, file: IO[Any], version: int = ...) -> None: ...
def load(file: IO[Any]) -> Any: ...
def dumps(value: Any, version: int = ...) -> str: ...
def loads(string: str) -> Any: ...
