from typing import Any

LONG_PASSWORD = ...  # type: Any
FOUND_ROWS = ...  # type: Any
LONG_FLAG = ...  # type: Any
CONNECT_WITH_DB = ...  # type: Any
NO_SCHEMA = ...  # type: Any
COMPRESS = ...  # type: Any
ODBC = ...  # type: Any
LOCAL_FILES = ...  # type: Any
IGNORE_SPACE = ...  # type: Any
PROTOCOL_41 = ...  # type: Any
INTERACTIVE = ...  # type: Any
SSL = ...  # type: Any
IGNORE_SIGPIPE = ...  # type: Any
TRANSACTIONS = ...  # type: Any
SECURE_CONNECTION = ...  # type: Any
MULTI_STATEMENTS = ...  # type: Any
MULTI_RESULTS = ...  # type: Any
CAPABILITIES = ...  # type: Any
