from typing import Any
from base64 import encodestring as encodebytes

from six.moves import http_client

expanduser = ...  # type: Any
StandardError = ...  # type: Any
long_type = ...  # type: Any
unquote_str = ...  # type: Any
parse_qs_safe = ...  # type: Any
