from typing import Any

NO = ...  # type: Any
YES = ...  # type: Any
TRUE = ...  # type: Any
FALSE = ...  # type: Any
ON = ...  # type: Any
OFF = ...  # type: Any
N = ...  # type: Any
S = ...  # type: Any
W = ...  # type: Any
E = ...  # type: Any
NW = ...  # type: Any
SW = ...  # type: Any
NE = ...  # type: Any
SE = ...  # type: Any
NS = ...  # type: Any
EW = ...  # type: Any
NSEW = ...  # type: Any
CENTER = ...  # type: Any
NONE = ...  # type: Any
X = ...  # type: Any
Y = ...  # type: Any
BOTH = ...  # type: Any
LEFT = ...  # type: Any
TOP = ...  # type: Any
RIGHT = ...  # type: Any
BOTTOM = ...  # type: Any
RAISED = ...  # type: Any
SUNKEN = ...  # type: Any
FLAT = ...  # type: Any
RIDGE = ...  # type: Any
GROOVE = ...  # type: Any
SOLID = ...  # type: Any
HORIZONTAL = ...  # type: Any
VERTICAL = ...  # type: Any
NUMERIC = ...  # type: Any
CHAR = ...  # type: Any
WORD = ...  # type: Any
BASELINE = ...  # type: Any
INSIDE = ...  # type: Any
OUTSIDE = ...  # type: Any
SEL = ...  # type: Any
SEL_FIRST = ...  # type: Any
SEL_LAST = ...  # type: Any
END = ...  # type: Any
INSERT = ...  # type: Any
CURRENT = ...  # type: Any
ANCHOR = ...  # type: Any
ALL = ...  # type: Any
NORMAL = ...  # type: Any
DISABLED = ...  # type: Any
ACTIVE = ...  # type: Any
HIDDEN = ...  # type: Any
CASCADE = ...  # type: Any
CHECKBUTTON = ...  # type: Any
COMMAND = ...  # type: Any
RADIOBUTTON = ...  # type: Any
SEPARATOR = ...  # type: Any
SINGLE = ...  # type: Any
BROWSE = ...  # type: Any
MULTIPLE = ...  # type: Any
EXTENDED = ...  # type: Any
DOTBOX = ...  # type: Any
UNDERLINE = ...  # type: Any
PIESLICE = ...  # type: Any
CHORD = ...  # type: Any
ARC = ...  # type: Any
FIRST = ...  # type: Any
LAST = ...  # type: Any
BUTT = ...  # type: Any
PROJECTING = ...  # type: Any
ROUND = ...  # type: Any
BEVEL = ...  # type: Any
MITER = ...  # type: Any
MOVETO = ...  # type: Any
SCROLL = ...  # type: Any
UNITS = ...  # type: Any
PAGES = ...  # type: Any
